function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var useScrollOnEdges = function useScrollOnEdges(props) {
  if (props === void 0) {
    props = undefined;
  }

  var _ref = props || {},
      _ref$canAnimate = _ref.canAnimate,
      canAnimate = _ref$canAnimate === void 0 ? true : _ref$canAnimate,
      _ref$edgeSize = _ref.edgeSize,
      edgeSize = _ref$edgeSize === void 0 ? 25 : _ref$edgeSize,
      _ref$scrollSpeed = _ref.scrollSpeed,
      scrollSpeed = _ref$scrollSpeed === void 0 ? 12 : _ref$scrollSpeed;

  var timer = React.useRef();

  function handleMousemove(event) {
    if (!canAnimate) {
      return;
    }

    var rect = event.currentTarget.getBoundingClientRect();
    var el = event.currentTarget;
    var viewportX = event.clientX - rect.left;
    var viewportY = event.clientY - rect.top;
    var viewportWidth = rect.width;
    var viewportHeight = rect.height;
    var edgeTop = edgeSize;
    var edgeLeft = edgeSize;
    var edgeBottom = viewportHeight - edgeSize;
    var edgeRight = viewportWidth - edgeSize;
    var isInLeftEdge = viewportX < edgeLeft;
    var isInRightEdge = viewportX > edgeRight;
    var isInTopEdge = viewportY < edgeTop;
    var isInBottomEdge = viewportY > edgeBottom;

    if (!(isInLeftEdge || isInRightEdge || isInTopEdge || isInBottomEdge)) {
      stopAnimmation();
    } else {
      startAnimation();
    }

    function animateScrolling(time) {
      if (shouldScroll()) {
        timer.current = undefined;
        startAnimation();
      } else {
        stopAnimmation();
      }
    }

    function startAnimation() {
      if (!timer.current) {
        timer.current = window.requestAnimationFrame(animateScrolling);
      }
    }

    function stopAnimmation() {
      if (timer.current) {
        window.cancelAnimationFrame(timer.current);
        timer.current = undefined;
      }
    }

    var maxScrollX = el.scrollWidth - el.clientWidth;
    var maxScrollY = el.scrollHeight - el.clientHeight;

    function shouldScroll() {
      var currentScrollX = el.scrollLeft;
      var currentScrollY = el.scrollTop;
      var canScrollUp = currentScrollY > 0;
      var canScrollDown = currentScrollY < maxScrollY;
      var canScrollLeft = currentScrollX > 0;
      var canScrollRight = currentScrollX < maxScrollX;
      var nextScrollX = currentScrollX;
      var nextScrollY = currentScrollY;

      if (isInLeftEdge && canScrollLeft) {
        nextScrollX = nextScrollX - scrollSpeed;
      } else if (isInRightEdge && canScrollRight) {
        nextScrollX = nextScrollX + scrollSpeed;
      }

      if (isInTopEdge && canScrollUp) {
        nextScrollY = nextScrollY - scrollSpeed;
      } else if (isInBottomEdge && canScrollDown) {
        nextScrollY = nextScrollY + scrollSpeed;
      }

      nextScrollX = Math.max(0, Math.min(maxScrollX, nextScrollX));
      nextScrollY = Math.max(0, Math.min(maxScrollY, nextScrollY));

      if (nextScrollX !== currentScrollX || nextScrollY !== currentScrollY) {
        el.scrollTo(nextScrollX, nextScrollY);
        nextScrollY = 0;
        return true;
      } else {
        return false;
      }
    }
  }

  function getEdgeScrollingProps(elementProps) {
    var callAllFns = function callAllFns() {
      for (var _len = arguments.length, fns = new Array(_len), _key = 0; _key < _len; _key++) {
        fns[_key] = arguments[_key];
      }

      return function () {
        for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        return fns.forEach(function (fn) {
          return fn && fn.apply(void 0, args);
        });
      };
    };

    return _extends({}, elementProps, {
      style: _extends({
        overflow: 'scroll'
      }, elementProps && elementProps.style),
      onMouseMove: callAllFns(elementProps && elementProps.onMouseMove, handleMousemove)
    });
  }

  return getEdgeScrollingProps;
};

module.exports = useScrollOnEdges;
//# sourceMappingURL=index.js.map
